import { ASTPath, Collection, JSCodeshift, JSXAttribute } from 'jscodeshift';
import { ButtonVariant, ButtonTone, OldVariant } from '../../../../src/types';

type VariantMapperType = {
  [key in OldVariant]: {
    variant: ButtonVariant;
    tone?: ButtonTone;
  };
};

const oldVariantToNewVariantAndTone: VariantMapperType = {
  default: {
    variant: 'default',
  },
  primary: {
    variant: 'emphasized',
    tone: 'default',
  },
  critical: {
    variant: 'emphasized',
    tone: 'critical',
  },
  success: {
    variant: 'emphasized',
    tone: 'success',
  },
  tertiary: {
    variant: 'plain',
  },
};

export default function updateVariant(j: JSCodeshift, source: Collection) {
  source.findJSXElements('Button').forEach((element) => {
    const attributes = (element.value.openingElement.attributes ??
      []) as unknown as ASTPath<JSXAttribute>[];
    const variantAttribute = attributes.find(
      (attribute) => attribute?.name?.name === 'variant',
    );

    if (variantAttribute == null) {
      return;
    }
    const valueType = variantAttribute.value.type as string;
    const isLiteral = valueType === 'Literal' || valueType === 'StringLiteral';

    if (!isLiteral) {
      return;
    }

    const value = variantAttribute.value.value as unknown as ButtonVariant;

    const newProps = oldVariantToNewVariantAndTone[value as OldVariant];

    if (newProps == null) {
      return null;
    }

    variantAttribute.value.value = newProps.variant as any;

    if (newProps.tone) {
      // eslint-disable-next-line no-param-reassign
      element.node.openingElement.attributes = [
        ...(attributes as any),
        j.jsxAttribute(j.jsxIdentifier('tone'), j.stringLiteral(newProps.tone)),
      ];
    }
  });
}
